(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.rek = factory());
}(this, (function () { 'use strict';

  function FetchError(response, body) {
    this.name = 'FetchError';
    this.message = response.statusText;
    this.status = response.status;
    this.body = body;
    this.response = response;
    this.stack = new Error().stack;
  }

  var requestMethods = ['delete', 'get', 'head'];
  var dataMethods = ['patch', 'post', 'put'];
  var responseTypes = {
    arrayBuffer: '*/*',
    blob: '*/*',
    formData: 'multipart/form-data',
    json: 'application/json',
    text: 'text/*'
  };
  function factory(defaults, api) {
    var _this = this;

    var fetch = api.fetch,
        FormData = api.FormData,
        Headers = api.Headers,
        URL = api.URL,
        URLSearchParams = api.URLSearchParams;

    function makeRequest(url, options) {
      return fetch(url, options).then(function (res) {
        if (!res.ok) {
          return res.text().then(function (text) {
            try {
              return JSON.parse(text);
            } catch (_unused) {
              return text;
            }
          }).catch(function () {}).then(function (body) {
            throw new FetchError(res, body);
          });
        }

        return res;
      });
    }

    function rek(url, options) {
      options = Object.assign({}, defaults, options);

      if (options.baseUrl) {
        url = new URL(url, options.baseUrl).href;
      }

      if (options.searchParams) {
        url = url.split('?')[0] + '?' + new URLSearchParams(options.searchParams);
      }

      var headers = options.headers = new Headers(Object.assign({}, defaults.headers, options.headers));
      var body = options.body;

      if (typeof body === 'object') {
        var prototype = Object.getPrototypeOf(body);

        if (prototype === null || prototype === Object.prototype) {
          if (!headers.has('content-type')) {
            headers.set('content-type', 'application/json');
          }

          if (headers.get('content-type').indexOf('application/json') > -1) {
            options.body = JSON.stringify(body);
          }
        } else if (FormData && body instanceof FormData || body instanceof URLSearchParams) {
          headers.delete('content-type');
        }
      }

      if (options.response === false) return makeRequest(url, options);
      var response = options.response || 'json';
      if (!(response in responseTypes)) throw new Error('Unknown response type: ' + response);
      headers.set('accept', responseTypes[response]);
      return makeRequest(url, options).then(function (res) {
        return res.status === 204 ? null : res[response]();
      });
    }

    requestMethods.forEach(function (method) {
      rek[method] = function (url, options) {
        return rek(url, Object.assign({}, options, {
          method: method.toUpperCase()
        }));
      };
    });
    dataMethods.forEach(function (method) {
      rek[method] = function (url, body, options) {
        return rek(url, Object.assign({}, options, {
          body: body,
          method: method.toUpperCase()
        }));
      };
    });

    rek.extend = function (newDefaults, newApi) {
      return factory.apply(_this, typeof newDefaults === 'function' ? newDefaults(defaults, api) : [Object.assign({}, defaults, newDefaults), Object.assign({}, api, newApi)]);
    };

    return rek;
  }

  var browser = factory({
    credentials: 'same-origin'
  }, self);

  return browser;

})));
//# sourceMappingURL=rek.umd.es5.js.map
